import pytest
from appium import webdriver


@pytest.fixture(autouse=True)
def app():
    global app_driver
    # APP定义运行环境
    desired_caps = {
        'deviceName': 'YAL_AL10',
        'automationName': 'appium',
        'platformName': 'Android',
        'platformVersion': '10.0',
        'appPackage': 'com.android.calculator2',
        'appActivity': '.Calculator',
    }
    app_driver = webdriver.Remote('http://localhost:4723/wd/hub', desired_caps)
    return app_driver


@pytest.fixture(autouse=True)
def app_close():
    yield app_driver
    app_driver.quit()



import time

LOCATOR_LIST = [
    "text",
    "textContains",
    "textMatches",
    "textStartsWith",
    "className",
    "classNameMatches",
    "description",
    "descriptionContains",
    "descriptionMatches",
    "descriptionStartsWith",
    "checkable",
    "checked",
    "clickable",
    "longClickable",
    "scrollable",
    "enabled",
    "focusable",
    "focused",
    "selected",
    "packageName",
    "packageNameMatches",
    "resourceId",
    "resourceIdMatches",
    "index",
    "instance",
]


class Page(object):

    def __init__(self, dr):
        self.driver = dr


class PageElement(object):

    driver = None

    def __init__(self, timeout=10, describe=None, **kwargs):
        self.time_out = timeout
        if not kwargs:
            raise ValueError("Please specify a locator")
        self.kwargs = kwargs
        self.k, self.v = next(iter(kwargs.items()))

        if self.k not in LOCATOR_LIST:
            raise KeyError("Element positioning of type '{}' is not supported.".format(self.k))

    def __get__(self, instance, owner):
        if instance is None:
            return None
        global driver
        driver = instance.driver
        return self

    def click(self):
        """
        click element
        """
        global driver
        for i in range(self.time_out):
            if driver(**self.kwargs).exists:
                break
            else:
                time.sleep(1)
        else:
            raise NameError("No corresponding element was found")
        driver(**self.kwargs).click()

    def set_text(self, text):
        """
        input text
        :param text:
        """
        global driver
        for i in range(self.time_out):
            if driver(**self.kwargs).exists:
                break
            else:
                time.sleep(1)
        else:
            raise NameError("No corresponding element was found")
        driver(**self.kwargs).set_text(text=text)

    def send_keys(self, text):
        """
        alias of set_text
        :param text:
        """
        return self.set_text(text)

    def clear_text(self):
        """
        Clear the text
        """
        return self.set_text(None)

    def get_text(self):
        """
        get element text
        """
        global driver
        for i in range(self.time_out):
            if driver(**self.kwargs).exists:
                break
            else:
                time.sleep(1)
        else:
            raise NameError("No corresponding element was found")
        return driver(**self.kwargs).get_text()

    def bounds(self):
        """
        Returns the element coordinate position
        :return: left_top_x, left_top_y, right_bottom_x, right_bottom_y
        """
        global driver
        for i in range(self.time_out):
            if driver(**self.kwargs).exists:
                break
            else:
                time.sleep(1)
        else:
            raise NameError("No corresponding element was found")
        return driver(**self.kwargs).bounds()

    def center(self):
        """
        Returns the center coordinates of the element
        return: center point (x, y)
        """
        global driver
        for i in range(self.time_out):
            if driver(**self.kwargs).exists:
                break
            else:
                time.sleep(1)
        else:
            raise NameError("No corresponding element was found")
        return driver(**self.kwargs).center()

    def swipe(self, direction, steps=1):
        """
        Performs the swipe action on the UiObject.
        Swipe from center
        :param direction: "left", "right", "up", "down"
        :param steps: move steps, one step is about 5ms
        """
        global driver
        for i in range(self.time_out):
            if driver(**self.kwargs).exists:
                break
            else:
                time.sleep(1)
        else:
            raise NameError("No corresponding element was found")
        return driver(**self.kwargs).swipe(direction, steps)

    def wait(self):
        """
        Wait until UI Element exists or gone
        """
        global driver
        return driver(**self.kwargs).wait(exists=True, timeout=self.time_out)



import warnings
import platform
from time import sleep
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.select import Select
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.common.exceptions import WebDriverException
from appium.webdriver.common.mobileby import MobileBy
from poium.common.exceptions import PageElementError
from poium.common.exceptions import FindElementTypesError
from poium.common import logging
from func_timeout import func_set_timeout
from func_timeout.exceptions import FunctionTimedOut
from poium.config import Browser


# Map PageElement constructor arguments to webdriver locator enums
LOCATOR_LIST = {
    # selenium
    'css': By.CSS_SELECTOR,
    'id_': By.ID,
    'name': By.NAME,
    'xpath': By.XPATH,
    'link_text': By.LINK_TEXT,
    'partial_link_text': By.PARTIAL_LINK_TEXT,
    'tag': By.TAG_NAME,
    'class_name': By.CLASS_NAME,
    # appium
    'ios_uiautomation': MobileBy.IOS_UIAUTOMATION,
    'ios_predicate': MobileBy.IOS_PREDICATE,
    'ios_class_chain': MobileBy.IOS_CLASS_CHAIN,
    'android_uiautomator': MobileBy.ANDROID_UIAUTOMATOR,
    'android_viewtag': MobileBy.ANDROID_VIEWTAG,
    'android_data_matcher': MobileBy.ANDROID_DATA_MATCHER,
    'android_view_matcher': MobileBy.ANDROID_VIEW_MATCHER,
    'windows_uiautomation': MobileBy.WINDOWS_UI_AUTOMATION,
    'accessibility_id': MobileBy.ACCESSIBILITY_ID,
    'image': MobileBy.IMAGE,
    'custom': MobileBy.CUSTOM,
}


class PageObject(object):
    """
    Page Object pattern.
    """

    def __init__(self, driver, url=None):
        """
        :param driver: `selenium.webdriver.WebDriver` Selenium webdriver instance
        :param url: `str`
        Root URI to base any calls to the ``PageObject.get`` method. If not defined
        in the constructor it will try and look it from the webdriver object.
        """
        self.driver = driver
        self.root_uri = url if url else getattr(self.driver, 'url', None)

    def get(self, uri):
        """
        :param uri:  URI to GET, based off of the root_uri attribute.
        """
        root_uri = self.root_uri or ''
        self.driver.get(root_uri + uri)
        self.driver.implicitly_wait(5)


class Element(object):
    """
    Returns an element object
    """

    def __init__(self, timeout=5, describe="undefined", index=0, **kwargs):
        self.timeout = timeout
        self.index = index
        self.desc = describe
        if not kwargs:
            raise ValueError("Please specify a locator")
        if len(kwargs) > 1:
            raise ValueError("Please specify only one locator")
        self.kwargs = kwargs
        self.k, self.v = next(iter(kwargs.items()))

        if self.k not in LOCATOR_LIST.keys():
            raise FindElementTypesError("Element positioning of type '{}' is not supported.".format(self.k))

    def __get__(self, instance, owner):
        if instance is None:
            return None

        Browser.driver = instance.driver
        return self

    def __set__(self, instance, value):
        self.__get__(instance, instance.__class__)
        self.send_keys(value)

    @func_set_timeout(0.5)
    def __elements(self, key, vlaue):
        elems = Browser.driver.find_elements(by=key, value=vlaue)
        return elems

    def __find_element(self, elem):
        """
        Find if the element exists.
        """
        for i in range(self.timeout):
            try:
                elems = self.__elements(elem[0], elem[1])
            except FunctionTimedOut:
                elems = []

            if len(elems) == 1:
                logging.info("✅ Find element: {by}={value} ".format(
                    by=elem[0], value=elem[1]))
                break
            elif len(elems) > 1:
                logging.info("❓ Find {n} elements through: {by}={value}".format(
                    n=len(elems), by=elem[0], value=elem[1]))
                break
            else:
                sleep(1)
        else:
            error_msg = "❌ Find 0 elements through: {by}={value}".format(by=elem[0], value=elem[1])
            logging.error(error_msg)
            raise NoSuchElementException(error_msg)

    def __get_element(self, by, value):
        """
        Judge element positioning way, and returns the element.
        """

        # selenium
        if by == "id_":
            self.__find_element((By.ID, value))
            elem = Browser.driver.find_elements_by_id(value)[self.index]
        elif by == "name":
            self.__find_element((By.NAME, value))
            elem = Browser.driver.find_elements_by_name(value)[self.index]
        elif by == "class_name":
            self.__find_element((By.CLASS_NAME, value))
            elem = Browser.driver.find_elements_by_class_name(value)[self.index]
        elif by == "tag":
            self.__find_element((By.TAG_NAME, value))
            elem = Browser.driver.find_elements_by_tag_name(value)[self.index]
        elif by == "link_text":
            self.__find_element((By.LINK_TEXT, value))
            elem = Browser.driver.find_elements_by_link_text(value)[self.index]
        elif by == "partial_link_text":
            self.__find_element((By.PARTIAL_LINK_TEXT, value))
            elem = Browser.driver.find_elements_by_partial_link_text(value)[self.index]
        elif by == "xpath":
            self.__find_element((By.XPATH, value))
            elem = Browser.driver.find_elements_by_xpath(value)[self.index]
        elif by == "css":
            self.__find_element((By.CSS_SELECTOR, value))
            elem = Browser.driver.find_elements_by_css_selector(value)[self.index]

        # appium
        elif by == "ios_uiautomation":
            self.__find_element((MobileBy.IOS_UIAUTOMATION, value))
            elem = Browser.driver.find_elements_by_ios_uiautomation(value)[self.index]
        elif by == "ios_predicate":
            self.__find_element((MobileBy.IOS_PREDICATE, value))
            elem = Browser.driver.find_elements_by_ios_predicate(value)[self.index]
        elif by == "ios_class_chain":
            self.__find_element((MobileBy.IOS_CLASS_CHAIN, value))
            elem = Browser.driver.find_elements_by_ios_class_chain(value)[self.index]
        elif by == "android_uiautomator":
            self.__find_element((MobileBy.ANDROID_UIAUTOMATOR, value))
            elem = Browser.driver.find_elements_by_android_uiautomator(value)[self.index]
        elif by == "android_viewtag":
            self.__find_element((MobileBy.ANDROID_VIEWTAG, value))
            elem = Browser.driver.find_elements_by_android_viewtag(value)[self.index]
        elif by == "android_data_matcher":
            self.__find_element((MobileBy.ANDROID_DATA_MATCHER, value))
            elem = Browser.driver.find_elements_by_android_data_matcher(value)[self.index]
        elif by == "accessibility_id":
            self.__find_element((MobileBy.ACCESSIBILITY_ID, value))
            elem = Browser.driver.find_elements_by_accessibility_id(value)[self.index]
        elif by == "android_view_matcher":
            self.__find_element((MobileBy.ANDROID_VIEW_MATCHER, value))
            elem = Browser.driver.find_elements_by_android_view_matcher(value)[self.index]
        elif by == "windows_uiautomation":
            self.__find_element((MobileBy.WINDOWS_UI_AUTOMATION, value))
            elem = Browser.driver.find_elements_by_windows_uiautomation(value)[self.index]
        elif by == "image":
            self.__find_element((MobileBy.IMAGE, value))
            elem = Browser.driver.find_elements_by_image(value)[self.index]
        elif by == "custom":
            self.__find_element((MobileBy.CUSTOM, value))
            elem = Browser.driver.find_elements_by_custom(value)[self.index]
        else:
            raise FindElementTypesError(
                "Please enter the correct targeting elements")
        if Browser.show is True:
            try:
                style_red = 'arguments[0].style.border="2px solid #FF0000"'
                style_blue = 'arguments[0].style.border="2px solid #00FF00"'
                style_null = 'arguments[0].style.border=""'

                for _ in range(2):
                    Browser.driver.execute_script(style_red, elem)
                    sleep(0.1)
                    Browser.driver.execute_script(style_blue, elem)
                    sleep(0.1)
                Browser.driver.execute_script(style_blue, elem)
                sleep(0.5)
                Browser.driver.execute_script(style_null, elem)
            except WebDriverException:
                pass

        return elem

    def clear(self):
        """Clears the text if it's a text entry element."""
        elem = self.__get_element(self.k, self.v)
        logging.info("clear element: {}".format(self.desc))
        elem.clear()

    def send_keys(self, value):
        """
        Simulates typing into the element.
        """
        elem = self.__get_element(self.k, self.v)
        logging.info("🖋 input element: {}".format(self.desc))
        elem.send_keys(value)

    def click(self):
        """Clicks the element."""
        elem = self.__get_element(self.k, self.v)
        logging.info("🖱 click element: {}".format(self.desc))
        elem.click()

    def submit(self):
        """Submits a form."""
        elem = self.__get_element(self.k, self.v)
        logging.info("submit element: {}".format(self.desc))
        elem.submit()

    @property
    def tag_name(self):
        """This element's ``tagName`` property."""
        elem = self.__get_element(self.k, self.v)
        return elem.tag_name

    @property
    def text(self):
        """Clears the text if it's a text entry element."""
        elem = self.__get_element(self.k, self.v)
        return elem.text

    @property
    def size(self):
        """The size of the element."""
        elem = self.__get_element(self.k, self.v)
        return elem.size

    def get_property(self, name):
        """
        Gets the given property of the element.
        """
        elem = self.__get_element(self.k, self.v)
        return elem.get_property(name)

    def get_attribute(self, name):
        """Gets the given attribute or property of the element."""
        elem = self.__get_element(self.k, self.v)
        return elem.get_attribute(name)

    def is_displayed(self):
        """Whether the element is visible to a user."""
        elem = self.__get_element(self.k, self.v)
        return elem.is_displayed()

    def is_selected(self):
        """
        Returns whether the element is selected.

        Can be used to check if a checkbox or radio button is selected.
        """
        elem = self.__get_element(self.k, self.v)
        return elem.is_selected()

    def is_enabled(self):
        """Returns whether the element is enabled."""
        elem = self.__get_element(self.k, self.v)
        return elem.is_enabled()

    def switch_to_frame(self):
        """
        selenium API
        Switches focus to the specified frame
        """
        elem = self.__get_element(self.k, self.v)
        Browser.driver.switch_to.frame(elem)

    def move_to_element(self):
        """
        selenium API
        Moving the mouse to the middle of an element
        """
        elem = self.__get_element(self.k, self.v)
        ActionChains(Browser.driver).move_to_element(elem).perform()

    def click_and_hold(self):
        """
        selenium API
        Holds down the left mouse button on an element.
        """
        elem = self.__get_element(self.k, self.v)
        ActionChains(Browser.driver).click_and_hold(elem).perform()

    def double_click(self):
        """
        selenium API
        Holds down the left mouse button on an element.
        """
        elem = self.__get_element(self.k, self.v)
        ActionChains(Browser.driver).double_click(elem).perform()

    def context_click(self):
        """
        selenium API
        Performs a context-click (right click) on an element.
        """
        elem = self.__get_element(self.k, self.v)
        ActionChains(Browser.driver).context_click(elem).perform()

    def drag_and_drop_by_offset(self, x, y):
        """
        selenium API
        Holds down the left mouse button on the source element,
           then moves to the target offset and releases the mouse button.
        :param x: X offset to move to.
        :param y: Y offset to move to.
        """
        elem = self.__get_element(self.k, self.v)
        ActionChains(Browser.driver).drag_and_drop_by_offset(elem, xoffset=x, yoffset=y).perform()

    def refresh_element(self, timeout=10):
        """
        selenium API
        Refreshes the current page, retrieve elements.
        """
        try:
            timeout_int = int(timeout)
        except TypeError:
            raise ValueError("Type 'timeout' error, must be type int() ")

        elem = self.__get_element(self.k, self.v)
        for i in range(timeout_int):
            if elem is not None:
                try:
                    elem
                except StaleElementReferenceException:
                    Browser.driver.refresh()
                else:
                    break
            else:
                sleep(1)
        else:
            raise TimeoutError("stale element reference: element is not attached to the page document.")

    def select_by_value(self, value):
        """
        selenium API
        Select all options that have a value matching the argument. That is, when given "foo" this
           would select an option like:

           <option value="foo">Bar</option>

           :Args:
            - value - The value to match against

           throws NoSuchElementException If there is no option with specisied value in SELECT
        """
        select_elem = self.__get_element(self.k, self.v)
        Select(select_elem).select_by_value(value)

    def select_by_index(self, index):
        """
        selenium API
        Select the option at the given index. This is done by examing the "index" attribute of an
           element, and not merely by counting.

           :Args:
            - index - The option at this index will be selected

           throws NoSuchElementException If there is no option with specisied index in SELECT
        """
        select_elem = self.__get_element(self.k, self.v)
        Select(select_elem).select_by_index(index)

    def select_by_visible_text(self, text):
        """
        selenium API
        Select all options that display text matching the argument. That is, when given "Bar" this
           would select an option like:

            <option value="foo">Bar</option>

           :Args:
            - text - The visible text to match against

            throws NoSuchElementException If there is no option with specisied text in SELECT
        """
        select_elem = self.__get_element(self.k, self.v)
        Select(select_elem).select_by_visible_text(text)

    def set_text(self, keys):
        """
        appium API
        Sends text to the element.
        """
        elem = self.__get_element(self.k, self.v)
        elem.set_text(keys)
        return self

    @property
    def location_in_view(self):
        """
        appium API
        Gets the location of an element relative to the view.
        Returns:
            dict: The location of an element relative to the view
        """
        elem = self.__get_element(self.k, self.v)
        return elem.location_in_view()

    def set_value(self, value):
        """
        appium API
        Set the value on this element in the application
        """
        elem = self.__get_element(self.k, self.v)
        elem.set_value(value)
        return self

    def input(self, text=""):
        elem = self.__get_element(self.k, self.v)
        elem.send_keys(text)

    def enter(self):
        elem = self.__get_element(self.k, self.v)
        elem.send_keys(Keys.ENTER)

    def select_all(self):
        elem = self.__get_element(self.k, self.v)
        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "a")
        else:
            elem.send_keys(Keys.CONTROL, "a")

    def cut(self):
        elem = self.__get_element(self.k, self.v)
        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "x")
        else:
            elem.send_keys(Keys.CONTROL, "x")

    def copy(self):
        elem = self.__get_element(self.k, self.v)
        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "c")
        else:
            elem.send_keys(Keys.CONTROL, "c")

    def paste(self):
        elem = self.__get_element(self.k, self.v)
        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "v")
        else:
            elem.send_keys(Keys.CONTROL, "v")

    def backspace(self):
        elem = self.__get_element(self.k, self.v)
        elem.send_keys(Keys.BACKSPACE)

    def delete(self):
        elem = self.__get_element(self.k, self.v)
        elem.send_keys(Keys.DELETE)

    def tab(self):
        elem = self.__get_element(self.k, self.v)
        elem.send_keys(Keys.TAB)

    def space(self):
        elem = self.__get_element(self.k, self.v)
        elem.send_keys(Keys.SPACE)


class Elements(object):
    """
    Returns a set of element objects
    """

    def __init__(self, context=False, describe="undefined", **kwargs):
        self.describe = describe
        if not kwargs:
            raise ValueError("Please specify a locator")
        if len(kwargs) > 1:
            raise ValueError("Please specify only one locator")
        self.k, self.v = next(iter(kwargs.items()))
        try:
            self.locator = (LOCATOR_LIST[self.k], self.v)
        except KeyError:
            raise FindElementTypesError("Element positioning of type '{}' is not supported. ".format(self.k))
        self.has_context = bool(context)

    def find(self, context):
        try:
            elems = context.find_elements(*self.locator)
        except NoSuchElementException:
            elems = []
        logging.info("✨ Find {n} elements through: {by}={value}, describe:{desc}".format(
            n=len(elems), by=self.k, value=self.v, desc=self.describe))
        return elems

    def __get__(self, instance, owner, context=None):
        if not instance:
            return None

        if not context and self.has_context:
            return lambda ctx: self.__get__(instance, owner, context=ctx)

        if not context:
            context = instance.driver

        return self.find(context)

    def __set__(self, instance, value):
        if self.has_context:
            raise PageElementError("Sorry, the set descriptor doesn't support elements with context.")
        elems = self.__get__(instance, instance.__class__)
        if not elems:
            raise PageElementError("Can't set value, no elements found")
        [elem.send_keys(value) for elem in elems]



import sys
import platform
import logging.handlers
from colorama import Fore, Style

_logger = logging.getLogger('poium')
_logger.setLevel(logging.DEBUG)
_handler = logging.StreamHandler(sys.stdout)
_handler.setFormatter(logging.Formatter('%(asctime)s %(message)s'))
_logger.addHandler(_handler)


def debug(msg):
    _logger.debug("DEBUG " + str(msg))


def info(msg):
    _logger.info(Fore.GREEN + "INFO " + str(msg) + Style.RESET_ALL)


def error(msg):
    _logger.error(Fore.RED + "ERROR " + str(msg) + Style.RESET_ALL)


def warn(msg):
    _logger.warning(Fore.YELLOW + "WARNING " + str(msg) + Style.RESET_ALL)


def _print(msg):
    _logger.debug(Fore.BLUE + "PRINT " + str(msg) + Style.RESET_ALL)


def set_level(level):
    """ 设置log级别

    :param level: logging.DEBUG, logging.INFO, logging.WARN, logging.ERROR
    :return:
    """
    _logger.setLevel(level)


def set_level_to_debug():
    _logger.setLevel(logging.DEBUG)


def set_level_to_info():
    _logger.setLevel(logging.INFO)


def set_level_to_warn():
    _logger.setLevel(logging.WARN)


def set_level_to_error():
    _logger.setLevel(logging.ERROR)



from poium.common import logging
from poium.common.exceptions import CSSFindElementError
from selenium.common.exceptions import JavascriptException


class CSSElement(object):
    """
    Only CSS selectors are supported.
    Please see help: http://www.w3school.com.cn/cssref/css_selectors.asp
    >> from page_objects import Page, CSSElements
    >> class MyPage(Page):
            input = CSSElements('.s_ipt')
            button = CSSElements('#su')
    """

    driver = None

    def __init__(self, css, index=None, describe=None):
        self.css = css
        if index is None:
            self.index = "0"
        else:
            self.index = str(index)
        self.desc = describe

    def __get__(self, instance, owner):
        if instance is None:
            return None
        global driver
        driver = instance.driver
        return self

    def _execute_javascript(self, js):
        """
        Run the javascript script
        """
        try:
            driver.execute_script(js)
        except JavascriptException:
            raise CSSFindElementError("Element discovery failure. ", js)

    def clear(self):
        """
        JavaScript API, Only support css positioning
        Clears the text if it's a text entry element, Only support css positioning
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                    elm.style.border="2px solid red";
                    elm.value = "";""".format(css=self.css, index=self.index)
        self._execute_javascript(js)

    def set_text(self, value):
        """
        JavaScript API, Only support css positioning
        Simulates typing into the element.
        :param value: input text
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                    elm.style.border="2px solid red";
                    elm.value = "{value}";""".format(css=self.css, index=self.index, value=value)
        self._execute_javascript(js)

    def click(self):
        """
        JavaScript API, Only support css positioning
        Click element.
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                   elm.style.border="2px solid red";
                   elm.click();""".format(css=self.css, index=self.index)
        self._execute_javascript(js)

    def click_display(self):
        """
        JavaScript API, Only support css positioning
        Click on the displayed element, otherwise skip it.
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = 'var elm = document.querySelector("' + self.css + '");' \
             ' if(elm != null){elm.style.border="2px solid red";elm.click();}'
        self._execute_javascript(js)

    def display(self):
        """
        JavaScript API, Only support css positioning
        Display hidden elements
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                    elm.style.display = "block";""".format(css=self.css, index=self.index)
        self._execute_javascript(js)

    def remove_attribute(self, attribute):
        """
        JavaScript API, Only support css positioning
        Remove element attribute, Only support css positioning
        :param attribute:
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                    elm.removeAttribute("{attr}");""".format(css=self.css, index=self.index, attr=attribute)
        self._execute_javascript(js)

    def set_attribute(self, attribute, value):
        """
        JavaScript API, Only support css positioning
        Setting element attribute, Only support css positioning
        :param attribute:
        :param value:
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                    elm.setAttribute("{attr}", "{value}");
                    """.format(css=self.css, index=self.index, attr=attribute, value=value)
        self._execute_javascript(js)

    def clear_style(self):
        """
        JavaScript API, Only support css positioning
        Clear element styles.
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                    elm.style="";""".format(css=self.css, index=self.index)
        self._execute_javascript(js)

    def clear_class(self):
        """
        JavaScript API, Only support css positioning
        Clear element class
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{index}];
                     elm.removeAttribute("class");""".format(css=self.css, index=self.index)
        self._execute_javascript(js)

    def inner_text(self, text):
        """
        JavaScript API, Only support css positioning
        The innerText property sets the text content of the specified element, Only support css positioning
        :param text: Inserted text
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{i}];
                     elm.innerText="{text}";""".format(css=self.css, i=self.index, text=text)
        self._execute_javascript(js)

    def remove_child(self, child=0):
        """
        JavaScript API, Only support css positioning
        Remove a node from the child node list
        :param child: child of the child node
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{i}];
                    elm.removeChild(elm.childNodes[{child}]);""".format(css=self.css, i=self.index, child=str(child))
        self._execute_javascript(js)

    def click_parent(self):
        """
        JavaScript API, Only support css positioning
        Click the parent element of the element
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{i}];
                    elm.parentElement.click();""".format(css=self.css, i=self.index)
        self._execute_javascript(js)

    def scroll(self, top=0, left=0):
        """
        JavaScript API, Only support css positioning
        scroll the div element on the page
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        if top is not 0:
            js = """var elm = document.querySelectorAll("{css}")[{i}];
                    elm.scrollTop={t};""".format(css=self.css, i=self.index, t=top)
            self._execute_javascript(js)
        if left is not 0:
            js = """var elm = document.querySelectorAll("{css}")[{i}];
                    elm.scrollLeft={l};""".format(css=self.css, i=self.index, l=left)
            self._execute_javascript(js)

    def move_to(self):
        """
        JavaScript API, Only support css positioning
        Move the mouse over the element
        """
        logging.info("Element of the current operation: {desc}".format(desc=self.desc))
        js = """var elm = document.querySelectorAll("{css}")[{i}];
                    elm.dispatchEvent(new Event("mouseover"));""".format(css=self.css, i=self.index)
        self._execute_javascript(js)




from poium import Page
from poium import Element


class BaiduIndexPage(Page):
    search_input = Element(name='wd', describe="搜索框")


def test_keys(browser):
    page = BaiduIndexPage(browser)
    page.get("https://www.baidu.com")

    # 输入 seldomm
    page.search_input.input("seldomm")

    # 删除多输入的一个m
    page.search_input.backspace()

    # 输入空格键+“教程”
    page.search_input.space()
    page.search_input.input("教程")

    # ctrl+a 全选输入框内容
    page.search_input.select_all()

    # ctrl+x 剪切输入框内容
    page.search_input.cut()

    # ctrl+v 粘贴内容到输入框
    page.search_input.paste()

    # 通过回车键来代替单击操作
    page.search_input.enter()



try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webdriver import WebDriver, WebElement
from selenium.common.exceptions import NoSuchElementException

from poium import Page, PageElement, PageElements

@pytest.fixture()
def webdriver():
    return mock.Mock(spec=WebDriver)


class TestConstructor:

    def test_page_element(self):
        elem_id = PageElement(id_='id')
        elem_name = PageElement(name='name')
        elem_class = PageElement(class_name='class')
        elem_tag = PageElement(tag='input')
        elem_link_text = PageElement(link_text='this_is_link')
        elem_partial_link_text = PageElement(partial_link_text='is_link')
        elem_xpath = PageElement(xpath='//*[@id="kk"]')
        elem_css = PageElement(css='#id')
        assert elem_id.k == 'id_'
        assert elem_name.k == "name"
        assert elem_class.k == "class_name"
        assert elem_tag.k == "tag"
        assert elem_link_text.k == "link_text"
        assert elem_partial_link_text.k == "partial_link_text"
        assert elem_xpath.k == "xpath"
        assert elem_css.k == "css"

    def test_page_element_bad_args(self):
        with pytest.raises(ValueError):
            PageElement()
        with pytest.raises(ValueError):
            PageElement(id_='foo', xpath='bar')


class TestGet:

    def test_get_unattached(self):
        assert PageElement(css='bar').__get__(None, None) is None


class TestSet:

    def test_set_multi(self, webdriver):
        class TestPage(Page):
            test_elems = PageElements(css='foo')

        page = TestPage(webdriver)
        elem1 = mock.Mock(spec=WebElement)
        elem2 = mock.Mock(spec=WebElement)
        webdriver.find_elements.return_value = [elem1, elem2]
        page.test_elems = "XXX"
        assert webdriver.find_elements.called_once_with(By.CSS_SELECTOR, 'foo')
        elem1.send_keys.assert_called_once_with('XXX')
        elem2.send_keys.assert_called_once_with('XXX')


class TestRootURI:

    def test_from_constructor(self, webdriver):
        class TestPage(Page):
            pass

        page = TestPage(webdriver, url="http://example.com")
        assert page.root_uri == 'http://example.com'

    def test_from_webdriver(self):
        class TestPage(Page):
            pass
        webdriver = mock.Mock(spec=WebDriver, url="http://example.com/foo")
        page = TestPage(webdriver)
        assert page.root_uri == 'http://example.com/foo'

    def test_get(self, webdriver):
        class TestPage(Page):
            pass
        page = TestPage(webdriver, url="http://example.com")
        page.get('/foo/bar')
        assert webdriver.get.called_once_with("http://example.com/foo/bar")

    def test_get_no_root(self, webdriver):
        class TestPage(Page):
            pass
        page = TestPage(webdriver)
        page.get('/foo/bar')
        assert webdriver.get.called_once_with("/foo/bar")


class TestTimeOut:

    def test_setting_time_out(self):
        elem = PageElement(css='foo', timeout=10)
        assert elem.k == "css"


class TestDescribe:

    def test_setting_describe(self):
        elem = PageElement(name='wd', describe="this is search input")
        assert elem.k == "name"


if __name__ == '__main__':
    pytest.main(["-v", "-s", "test_page_objects.py"])



from poium import Page, Element


class CalculatorPage(Page):
    number_1 = Element(id_="com.android.calculator2:id/digit_1")
    number_2 = Element(id_="com.android.calculator2:id/digit_2")
    add = Element(id_="com.android.calculator2:id/op_add")
    eq = Element(id_="com.android.calculator2:id/eq")


def test_calculator(app):
    page = CalculatorPage(app)
    page.number_1.click()
    page.add.click()
    page.number_2.click()
    page.eq.click()



import time


LOCATOR_LIST = [
    "id",
    "name",
    "text",
    "nameContains",
    "label",
    "xpath",
    "labelContains",
    "className"
]


class Page(object):

    def __init__(self, dr):
        self.driver = dr

    def bundle_id(self):
        """
        the session matched bundle id
        """
        return self.driver.bundle_id

    def locked(self):
        """
        returns locked status, true or false
        """
        return self.driver.locked()

    def lock(self):
        """
        Execute screen lock
        """
        return self.driver.lock()

    def unlock(self):
        """
        unlock screen, double press home
        """
        return self.driver.unlock()

    def battery_info(self):
        """
        Returns dict: (I do not known what it means)
        """
        return self.driver.battery_info()

    def device_info(self):
        """
        Returns dict: (I do not known what it means)
        """
        return self.driver.device_info()

    def app_current(self):
        """
        Current application information
        """
        return self.driver.app_current()

    def set_clipboard(self):
        """
        set clipboard
        """
        self.driver.set_clipboard()

    def window_size(self):
        """
        Gets the real resolution of the screen
        """
        multiple = self.driver.scale
        w, h = self.driver.window_size()
        return multiple * w, multiple * h

    def close(self):
        """
        close App
        """
        self.driver.close()

    def swipe(self, start, end, duration=0):
        """
        滑动
        start:
            list类型，起始坐标
        end:
            list类型，终点坐标
        """
        if type(start) is list or type(end) is list:
            self.driver.swipe(start[0], start[1], end[0], end[1], duration)
        else:
            raise TypeError("The argument must be a list")

    def swipe_left(self, pos, times=None):
        """
        滑向左边
        times:
            int类型， 滑动的次数
        """
        if times is None:
            times = 1
        if type(pos) is list:
            for i in range(times):
                self.swipe([pos[0], pos[1]], [pos[0] + 0.2, pos[1]])
        else:
            raise TypeError("The argument must be a list")

    def swipe_right(self, pos, times=False):
        """
        滑向右边
        times:
            int类型， 滑动的次数
        """
        if times is False:
            times = 1
        if type(pos) is list:
            for i in range(times):
                self.swipe([pos[0], pos[1]], [pos[0] - 0.2, pos[1]])
        else:
            raise TypeError("The argument must be a list")

    def swipe_up(self, times=False):
        """
        向上滑动
        times:
            int类型， 滑动的次数
        """
        if times is False:
            times = 1
        for i in range(times):
            self.swipe([0.5, 0.3], [0.5, 0.6])

    def swipe_down(self, times=False):
        """
        向下滑动
        times:
            int类型， 滑动的次数
        """
        if times is False:
            times = 1
        for i in range(times):
            self.swipe([0.5, 0.6], [0.5, 0.3])


class PageElement(object):

    driver = None

    def __init__(self, timeout=10, describe=None, **kwargs):
        self.time_out = timeout
        if not kwargs:
            raise ValueError("Please specify a locator")
        self.kwargs = kwargs
        self.k, self.v = next(iter(kwargs.items()))

        if self.k not in LOCATOR_LIST:
            raise KeyError("Element positioning of type '{}' is not supported.".format(self.k))

    def __get__(self, instance, owner):
        if instance is None:
            return None
        global driver
        driver = instance.driver
        return self

    def click(self):
        """
        点击元素
        """
        global driver
        for i in range(self.time_out):
            time.sleep(1)
            if driver(**self.kwargs).exists:
                break
        else:
            raise NameError("No corresponding element was found")
        driver(**self.kwargs).click()

    def set_text(self, value):
        """
        输入文本内容
         value:
              Must be of type string
        """
        global driver
        if type(value) is str:
            driver(**self.kwargs).set_text(value)

    def get_text(self):
        """
        获取元素对应的文本
        """
        global driver
        return driver(**self.kwargs).text

    def focus(self, position):
        """
        定位元素区域内的坐标
        :return:
        """
        global driver
        if type(position) is not list:
            raise NameError("The argument must be a list")
        elif position[0] > 1 or position[1] > 1:
            raise NameError("Coordinates range from 0 to 1")
        w, h = driver.window_size()
        rect = driver(**self.kwargs).bounds
        x = rect.x + rect.width * position[0]
        y = rect.y + rect.height * position[1]
        return x / w, y / h

    def focus_click(self, position):
        """
        点击元素范围内的某个位置
        :return:
        """
        if type(position) is not list:
            raise NameError("The argument must be a list")
        elif position[0] > 1 or position[1] > 1:
            raise NameError("Coordinates range from 0 to 1")
        x, y = self.focus(position=position)
        driver.click(x, y)

    def get_position(self, percentage=True):
        """
        获取元素坐标
        :return: 返回坐标百分比
        """
        global driver
        w, h = driver.window_size()
        rect = driver(**self.kwargs).bounds
        x = rect.x + rect.width / 2
        y = rect.y + rect.height / 2
        if percentage is True:
            return x / w, y / h
        elif percentage is False:
            return x, y

    def exists(self):
        """
        判断元素是否存在
        """
        global driver
        status = driver(**self.kwargs).exists
        return status

    def displayed(self):
        """
        判断元素是否存当前页面
        """
        global driver
        status = driver(**self.kwargs).displayed
        return status

    def scroll(self, direction='visible', distance=1.0):
        """
        滚动定位到对应的元素
        Args:
            direction (str): one of "visible", "up", "down", "left", "right"
            distance (float): swipe distance, only works when direction is not "visible"

        Raises:
            ValueError

        distance=1.0 means, element (width or height) multiply 1.0
        """
        global driver
        driver(**self.kwargs).scroll(direction=direction, distance=distance)

    def scroll_search(self, click=False):
        """
        滚动定位到对应的元素

        Args:
            click: 定位到元素后，是否点击
        """
        global driver
        driver(**self.kwargs).scroll()
        if click is True:
            self.click()

    def swipe(self, direction, times=1):
        """
        基于元素滑动

        Args:
            direction (str): 方向，只能填写'up', 'down', 'left', 'right'
            times (int): 滑动次数
        """
        global driver
        x, y = self.get_position()
        if direction in ['up', 'down', 'left', 'right']:
            for i in range(times):
                if direction is "up":
                    driver.swipe(x, 0.5, x, 0.6)
                elif direction is "down":
                    driver.swipe(x, 0.5, x, 0.4)
                elif direction is "left":
                    driver.swipe(x, y, x + 0.15, y)
                elif direction is "right":
                    driver.swipe(x, y, x - 0.15, y)
        else:
            raise NameError

    def swipe_down(self, times=1):
        """
        基于元素向下滑动

        Args:
            times（int）: 滑动次数
        """
        self.swipe(direction="down", times=times)

    def swipe_up(self, times=1):
        """
        基于元素向上滑动

        Args:
            times（int）: 滑动次数
        """
        self.swipe(direction="up", times=times)

    def swipe_left(self, times=1):
        """
        基于元素向左滑动

        Args:
            times（int）: 滑动次数
        """
        self.swipe(direction="left", times=times)

    def swipe_right(self, times=1):
        """
        基于元素向右滑动

        Args:
            times（int）: 滑动次数
        """
        self.swipe(direction="right", times=times)

    def tap_hold(self, duration=1.0):
        """
        Tap and hold for a moment

        Args:
            duration (float): seconds of hold time
        """
        global driver
        driver(**self.kwargs).tap_hold(duration=duration)

    def precision_move(self, height):
        """
        将元素滑动到想要的位置
        :param height:
        :return:
        """
        if 0 < height < 1:
            x, y = self.get_position()
            height_max = height + 0.05
            height_min = height - 0.05
            for i in range(10):
                if height_min <= y <= height_max:
                    break
                if 0 <= height - y < 0.3:
                    self.scroll(direction="up", distance=1)
                elif height - y >= 0.3:
                    self.swipe_down()
                elif -0.3 < height - y <= 0:
                    self.scroll(direction="down", distance=1)
                elif -0.3 >= height - y:
                    self.swipe_up()
                x, y = self.get_position()
        else:
            raise ValueError



"""
uiautomator2 Library test demo
https://github.com/openatx/uiautomator2
"""
import uiautomator2 as u2
from poium.u2 import Page, PageElement


class BBSPage(Page):
    search_input = PageElement(resourceId="com.meizu.flyme.flymebbs:id/kp", describe="搜索输入框")
    search_button = PageElement(resourceId="com.meizu.flyme.flymebbs:id/o2", describe="搜索按钮")
    search_result = PageElement(resourceId="com.meizu.flyme.flymebbs:id/a2a", describe="搜索结果")


d = u2.connect()
d.app_start("com.meizu.flyme.flymebbs")
page = BBSPage(d)

page.search_input.click()
page.search_input.set_text("flyme")
page.search_button.click()

result = page.search_result.get_text()
print(result)

d.app_stop("com.meizu.flyme.flymebbs")




from poium import Page
from poium import Element
from time import sleep


class BaiduPage(Page):
    setting = Element(css='#s-usersetting-top')
    search_setting = Element(css=".setpref")
    search_setting_hint = Element(css="#sugConf > span.item-desc")


class DataTimePage(Page):
    frame = Element(id_="iframe")
    date = Element(id_="appDate")
    year = Element(css=".dwwo", index=0)
    mouth = Element(css=".dwwo", index=1)
    day = Element(css=".dwwo", index=2)


def test_move_to_element(browser):
    """
    测试鼠标悬停
    :param browser:
    :return:
    """
    page = BaiduPage(browser)
    page.get("https://www.baidu.com")
    page.setting.click_and_hold()
    page.search_setting.click()
    sleep(2)
    hint = page.search_setting_hint.text
    assert "搜索框提示：" in hint


def test_drag_and_drop_by_offset(browser):
    """
    测试鼠标滑动日期控件
    :param browser:
    :return:
    """
    page = DataTimePage(browser)
    page.get("http://www.jq22.com/yanshi4976")

    page.frame.switch_to_frame()
    page.date.click()

    page.year.drag_and_drop_by_offset(0, 10)
    sleep(2)
    page.mouth.drag_and_drop_by_offset(0, 20)
    sleep(2)
    page.day.drag_and_drop_by_offset(0, 30)
    sleep(2)



from poium import Page, Element


class BaiduPage(Page):
    login_button = Element(link_text="登录")
    register_link = Element(link_text="立即注册")


def test_switch_windows(browser):
    page = BaiduPage(browser)
    page.get("https://www.baidu.com")

    # 当前窗口句柄
    current_handler = page.current_window_handle
    print(current_handler)

    # 打开新的注册窗口
    page.login_button.click()
    page.register_link.click()

    # 新窗口句柄
    new_handler = page.new_window_handle
    print(new_handler)

    # 所有窗口句柄
    all_handler = page.window_handles
    print(all_handler)

    # 切换窗口句柄
    page.switch_to_window(current_handler)
    page.switch_to_window(new_handler)




from poium import Page, CSSElement, Element
from time import sleep


class BaiduPage(Page):
    search_input = CSSElement("#kw", describe="百度搜索框")
    search_button = CSSElement("#su", describe="百度按钮")
    icp = CSSElement("#cp", describe="备案信息")
    search_key = CSSElement(".res-gap-right16", describe="")
    setting_dropdown_box = CSSElement("#s_user_name_menu", describe="设置下拉框")
    setting = CSSElement("#s-usersetting-top", describe="设置")


class SoPage(Page):
    setting = CSSElement("#hd_setting", describe="搜索设置")
    search = CSSElement("#search-button", describe="搜索按钮")


class RunoobPage(Page):
    iframe = Element(css="#iframeResult")
    div = CSSElement("html > body > div")


def test_move_to(browser):
    """
    鼠标悬停到元素上
    :param browser: 浏览器驱动
    :return:
    """
    page = BaiduPage(browser)
    page.get("https://www.baidu.com")
    page.set_window_size()
    page.setting.move_to()
    sleep(5)


def test_clear_input_click(browser):
    """
    清除/输入/点击
    :param browser: 浏览器驱动
    :return:
    """
    page = BaiduPage(browser)
    page.get("https://www.baidu.com")
    page.search_input.clear()
    page.search_input.set_text("poium")
    page.search_button.click()
    sleep(2)
    page.search_key.click_display()
    sleep(2)
    assert page.get_title == "poium_百度搜索"


def test_get_info(browser):
    """
    获取页面标题,URL,文本
    :param browser: 浏览器驱动
    :return:
    """
    page = BaiduPage(browser)
    page.get("https://www.baidu.com")
    sleep(2)
    title = page.get_title
    url = page.get_url
    assert "百度一下，你就知道" == title
    assert "www.baidu.com" in url


def test_get_attribute(browser):
    """
    元素属性修改/获取/删除
    :param browser: 浏览器驱动
    :return:
    """
    page = BaiduPage(browser)
    page.get("https://www.baidu.com")
    sleep(2)
    page.search_button.remove_attribute("class")
    page.search_input.set_attribute("type", "password")
    page.search_input.set_text("abc123")
    sleep(5)


def test_display(browser):
    """
    显示隐藏的元素
    :param browser: 浏览器驱动
    :return:
    """
    page = SoPage(browser)
    page.get("https://www.so.com")
    page.setting.display()
    sleep(5)


def test_clear_class(browser):
    """
    清除元素的class属性
    :param browser: 浏览器驱动
    :return:
    """
    page = SoPage(browser)
    page.get("https://www.so.com")
    page.search.clear_class()
    sleep(5)


def test_scroll(browser):
    """
    测试操作页面内嵌滚动条
    :param browser: 浏览器驱动
    :return:
    """
    page = RunoobPage(browser)
    page.get("https://www.runoob.com/try/try.php?filename=tryjsref_onscroll")
    page.iframe.switch_to_frame()
    page.div.scroll(top=100)
    sleep(5)



# coding=utf-8
import re
import ast
from setuptools import setup, find_packages

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('poium/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='poium',
    version=version,
    url='https://github.com/SeldomQA/poium',
    license='BSD',
    author='fnngj',
    author_email='fnngj@126.com',
    description='Selenium/appium-based Page Objects test library.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=['selenium>=3.14.1',
                      'Appium-Python-Client>=1.0.1',
                      'colorama>=0.4.3',
                      'func-timeout>=4.3.5'],
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    extras_require={
    },
)



import pytest
from selenium import webdriver


@pytest.fixture(scope='session', autouse=True)
def browser():
    global driver
    driver = webdriver.Chrome()
    return driver


@pytest.fixture(scope="session", autouse=True)
def browser_close():
    yield driver
    driver.quit()



from poium import Page
from poium import Element
from time import sleep


class SelectPage(Page):
    frame = Element(id_="iframeResult", describe="表单")
    select_elem = Element(xpath="//select", describe="选择框")


def test_select(browser):
    """
    测试选择框的操作
    :param browser:
    :return:
    """
    page = SelectPage(browser)

    page.get("http://www.w3school.com.cn/tiy/t.asp?f=html_select")
    page.frame.switch_to_frame()
    page.select_elem.select_by_value("saab")
    sleep(2)
    page.select_elem.select_by_index(2)
    sleep(2)
    page.select_elem.select_by_visible_text("Audi")
    sleep(2)

